/*This code simulates basic response of a CS5490.
Connect it to a HMIP-FSM Controller Board
Use without 230V CS5490 Board !!!!!!
https://statics.cirrus.com/pubs/proDatasheet/CS5490_F3.pdf
https://files2.elv.com/public/15/1522/152281/Internet/152281_bauanleitung.pdf
*/
#include <SoftwareSerial.h>
#define togglepin50hz 5
#define analoginpin 24  //ADC2
#define sensorpowersupplypin 4
#define rxPin 2
#define txPin 3

volatile char datareadybyte = B11100010;  //xx####### x= interrupt bits, if xx set to 1, HM controller will start to read measured values. Don't forget to clear this bits.
int x = 0;
long value = 0;
volatile char adcbuffer[4];

SoftwareSerial SerialS1(rxPin, txPin);
int c = 0;
ISR(TIMER1_COMPA_vect) {
  digitalWrite(togglepin50hz, !digitalRead(togglepin50hz));  //toggle  pin with 50hz for DO interface. not sure if required

  if (x == 400) {
    datareadybyte = B11100010;  //set interrupt bits every 5 seconds, this is refresh rate for data upload. shorter value can cause upload issues
    x = 0;
  }
  x++;


  if (x == 99)  // 40ms power on to measure something
  {
    digitalWrite(sensorpowersupplypin, true);
  }
  if (x == 101) {
    digitalWrite(sensorpowersupplypin, false);
  }

  if (x == 100) {
    value = analogRead(analoginpin);

    value = 20; // do your caluclations here
    Serial.print(value);
    value = value * 312;  //multiply your value with 312 to get correct value in app. I calculated it should be 312,6717557 for more accurate conversation
    adcbuffer[0] = value >> 24;
    adcbuffer[1] = value >> 16;
    adcbuffer[2] = value >> 8;
    adcbuffer[3] = value;
  }
}

void setup() {
  pinMode(togglepin50hz, OUTPUT);
  pinMode(sensorpowersupplypin, OUTPUT);
  digitalWrite(sensorpowersupplypin, false);  //switch off meausrement power supply
  Serial.begin(115200);
  pinMode(rxPin, INPUT);

  pinMode(txPin, OUTPUT);
  SerialS1.begin(115200);

  //50Hz Timer for DO interface, not sure if required.
  TCCR1A = 0;               //Reset Timer1 control Registor A
  bitClear(TCCR1B, WGM13);  //Set CTC mode
  bitSet(TCCR1B, WGM12);
  bitSet(TCCR1B, CS12);  //Set prescaler to 1024
  bitClear(TCCR1B, CS11);
  bitSet(TCCR1B, CS10);
  TCNT1 = 0;
  OCR1A = 156;
  bitSet(TIMSK1, OCIE1A);
  sei();
}

void loop() {
  if (Serial.available()) {
  }
  if (SerialS1.available()) {
    char inbyte = SerialS1.read();
    char outbyte = 0;
    //00 Register Read  //1 frame and 3 frame as answer
    //01 Register Write //4 frames
    //10 Page Select    //1 frame
    //11 Instruction    //1 frame

    //in case of write command ignore next 3 frames, but this code does not ignore and therefore checks all. I try to keep it simple.

    if (inbyte == B00010111)  //HM controller read interupt status register
    {
      outbyte = B00100001;  //xx###### x = interrupt bits, if xx set to 1, HM controller will start to read measured values. // 000x0000 x=1 trigger Epsilon interrupt
      SerialS1.print(outbyte);
      outbyte = B00100101;
      SerialS1.print(outbyte);
      outbyte = datareadybyte;
      SerialS1.print(outbyte);
      datareadybyte = B00100010;

    } else if (inbyte == B00110001)  //Epsilon, HM Controller can optionally read this data, didn't checked purpose, not sure if required
    {
      outbyte = B00010001;  //just guessed values, didn't checked specification
      SerialS1.print(outbyte);
      outbyte = B00111000;  //just guessed values, didn't checked specification
      SerialS1.print(outbyte);
      outbyte = B10111010;  //just guessed values, didn't checked specification
      SerialS1.print(outbyte);
    }

    else if (inbyte == B00000101)  //Power, this function sends data which are visible in HM App
    {
      /*
        // debug help:
        outbyte=B00110111; //128.....1  1Watt= 313 multiply your sensor value by 313, the result can be send as long value. 0....20000W worked for me
        Serial.print(outbyte);
        outbyte=B00001100; //32768 .....256
        Serial.print(outbyte);
        outbyte=B00000000; //65536 ...... 8388608
        Serial.print(outbyte);*/

      SerialS1.print(adcbuffer[3]);
      SerialS1.print(adcbuffer[2]);
      SerialS1.print(adcbuffer[1]);
    } else if (inbyte == B00000110)  //IRMS
    {
      outbyte = B00000000;
      SerialS1.print(outbyte);
      outbyte = B00011000;  //just guessed value for current, didn't checked specification. this value is checked by HM Controller to be in realistic range. this value works for me always. Too high value will trigger fault notification in app.
      SerialS1.print(outbyte);
      outbyte = B00000000;
      SerialS1.print(outbyte);

    } else if (inbyte == B00000111)  //VRMS
    {
      outbyte = B00000000;
      SerialS1.print(outbyte);
      outbyte = B00010000;  //guessed value for voltage, didn't checked specification. this value is checked by HM Controller to be in realistic range. this value works for me always. Too high value will trigger fault notification in app.
      SerialS1.print(outbyte);
      outbyte = B00000000;
      SerialS1.print(outbyte);

    } else if (inbyte == B00000000)  //empty request, occure during power on
    {
    } else  //unknow register, check CS5490 datasheet
    {
    }
    //Serial.println(inbyte);
  }
}